#!/bin/sh
trap "" HUP

APP_NAME="saas_logclient"
COMMON_FILE="/usr/logd/www/app/${APP_NAME}/bin/saas_logclient_common"
. ${COMMON_FILE}

TASK_TYPE=""
TASK_ARGSTR=""
SAAS_DOMAIN='saas.panabit.com'
SAAS_PORT='8376'
SSH_ENABLE=0
SSHRC_PASSWDFREE=0
SAAS_RELTYPE='saaslogclient'
CONF_DIR="${LOGROOT}/app/${APP_NAME}"
CONF_FILE="${CONF_DIR}/${APP_NAME}.conf"
rcfgres=`cat ${CONF_FILE}`
while read line
do
	key=`echo $line |cut -d'=' -f1`
	value=`echo $line |cut -d'=' -f2`
	eval "$key=\"$value\""
done << EOF
	$rcfgres
EOF

SAAS_PING="${LOGROOT}/app/${APP_NAME}/bin/saasping"
TASK_SCRIPT_PATH="${LOGROOT}/app/${APP_NAME}/bin/"
TASK_DIR="${LOGROOT}/app/${APP_NAME}/tasks/"
TASK_EXEC_SCRIPT="${LOGROOT}/app/${APP_NAME}/bin/saas_logclient_taskexec.sh"
SAAS_DEVINFO_FILE="${LOGROOT}/app/${APP_NAME}/devinfo.txt"
SAAS_SERVER_ONLINETAG_FILE="${LOGROOT}/app/${APP_NAME}/saas_server_online_tag"
LAST_KEYNO=""
CUR_OS=`uname`
osname=$(uname -r | grep -oE '(el|oe|an)[^ ]*' | awk -F'.' '{print $1}')
[ "`uname -m`" = "aarch64" ] && CUR_OS="Arm"
[ -n "${osname}" ] && [ "${osname:0:2}" = "el" ] && CUR_OS="CentOS"
[ -n "${osname}" ] && [ "${osname:0:2}" = "an" ] && CUR_OS="Anolis"
[ -n "${osname}" ] && [ "${osname:0:2}" = "oe" ] && CUR_OS="Euler"

mkdir -p ${TASK_DIR}
[ -e "${SAAS_SERVER_ONLINETAG_FILE}" ] && rm -f ${SAAS_SERVER_ONLINETAG_FILE}
[ -e "${SAAS_DEVINFO_FILE}" ] && rm -f ${SAAS_DEVINFO_FILE}
[ -e "${SAAS_DEVINFO_FILE}.all" ] && rm -f ${SAAS_DEVINFO_FILE}.all

loginkey_get()
{
	echo "id_rsa"
}

get_basic_info_free()
{
    cpu_count=0
    cpuper=0
    memsize=0
    memper=0
    hardsize=0
    hardper=0

    if [ -f "/tmp/cpustat.log" ]; then
        cmd=`cat /tmp/cpustat.log`
        cpu_count=$(echo $cmd | cut -d ' ' -f 1)
        cpuper=$(echo $cmd | cut -d ' ' -f 2)
        memsize=$(echo $cmd | cut -d ' ' -f 3)
        memper=$(echo $cmd | cut -d ' ' -f 4)
        hardsize=$(echo $cmd | cut -d ' ' -f 6)
        hardper=$(echo $cmd | cut -d ' ' -f 5)
    fi
    memsize=$(($memsize / 1024 / 1024 / 1024))
    hardper=`echo ${hardper} |sed 's/%//g'`
    echo "$cpu_count $cpuper $memsize $memper $hardsize $hardper"
}

get_basic_info_cluster()
{
    local nodes=0

    local res=`${PALOGEYE} node list`
    while read line
    do
        field1=$(echo $line | cut -d ' ' -f 1) 
        if [ ${field1} == "ipaddr" ]; then
	    continue
        fi
        nodes=$(($nodes + 1))
        cpu_count=$(echo $line | cut -d ' ' -f 5)
        cpuper=$(echo $line | cut -d ' ' -f 6)
        memsize=$(echo $line | cut -d ' ' -f 7)
        memper=$(echo $line | cut -d ' ' -f 8)
        hardsize=$(echo $line | cut -d ' ' -f 9)
        hardper=$(echo $line | cut -d ' ' -f 10) 
    done << EOF
	${res}
EOF

    if [ $nodes -eq 0 ]; then
        get_basic_info_free
    else
        cpuper=$(($cpuper / $nodes))
        memper=$(($memper / $nodes))
        hardper=$(($hardper / $nodes))
    fi
    echo "$cpu_count $cpuper $memsize $memper $hardsize $hardper"
}

get_basic_info()
{
    local version=`${PALOGEYE} receiver stat | grep version | awk -F "=" '{print $2}'`
    if [ ${version} -eq 0 ]; then
        # panalog free version
        get_basic_info_free
    elif [ ${version} -eq 1 ]; then
        # panalog cluster version
        get_basic_info_cluster
    else
        echo "0 0 0 0 0 0"
    fi
}

get_sys_info()
{
    systype="panalog_free" # Specifications and models

    local version=`${PALOGEYE} receiver stat | grep version | awk -F "=" '{print $2}'`
    if [ ${version} -eq 1 ]; then
        systype="panalog_cluster"
		sysinfores=`${PALOGEYE} key stat`
		eval "$sysinfores"
	else
		keyno=`${PALOGEYE} key get ver=free|awk '{print $2}'`
		start2=0
		end2=0
		upgrade_start2=0
		upgrade_end2=0
		nodecount=0
    fi

	

    echo "$version $keyno $start2 $end2 $upgrade_start2 $upgrade_end2 $nodecount"
}

get_soft_info()
{
	local res=`cat ${LOGROOT}/cretime.txt`
	while read line
	do
		key=`echo $line |cut -d'=' -f1`
		value=`echo $line |cut -d'=' -f2`
		eval "$key=\"$value\""
	done << EOF
		$res
EOF

	echo "$BUILDTIME $SOFTVER $OSTYPE $CORETYPE"
}

LAST_ptype=""
LAST_logsn65=""
LAST_logver=""
LAST_CUR_OS=""
LAST_oscore=""
LAST_cpustat_cpurate=""
LAST_bootime=""
LAST_cpustat_physmemM=""
LAST_keyinfo_start=0
LAST_keyinfo_end=0
LAST_palog_nodecount=0
LAST_keyinfo_freestart=0
LAST_palog_cpucnt=0
LAST_keyinfo_freeend=0
LAST_palog_hardrate=0
LAST_palog_hardsize=0
LAST_palog_memrate=0
LAST_softinfo_buildtime=""
LAST_softinfo_ver==""
LAST_softinfo_ostype=""
LAST_clicfg_sshrc=0
LAST_saasagent_ver=""

sshd_enable()
{
	# os=`uname`

	if [ "${CUR_OS}" = "Linux" ]; then
		if [ "$1" = "off" ]; then
			/etc/init.d/sshd stop >/dev/null 2>&1
		else
			/etc/init.d/sshd restart >/dev/null 2>&1
		fi
	elif [ "${CUR_OS}" = "FreeBSD" ]; then
		if [ "$1" = "off" ]; then
			service sshd stop >/dev/null 2>&1
			# /etc/rc.d/sshd onestop >/dev/null 2>&1
		else
			service sshd restart >/dev/null 2>&1
			# /etc/rc.d/sshd onerestart >/dev/null 2>&1
		fi
	else
		if [ "$1" = "off" ]; then
			systemctl stop sshd >/dev/null 2>&1
		else
			systemctl restart sshd >/dev/null 2>&1
		fi
	fi
}

sshd_status() {
    if [ "${CUR_OS}" = "FreeBSD" ]; then
        if service sshd onestatus >/dev/null 2>&1; then
            echo "1"
        else
            echo "0"
        fi
	elif [ "${CUR_OS}" = "CentOS" ] || [ "${CUR_OS}" = "Anolis" ] || [ "${CUR_OS}" = "Euler" ]; then
		if systemctl is-active --quiet sshd; then
			echo "1"
		else
			echo "0"
		fi
	else
		if /etc/init.d/sshd status >/dev/null 2>&1; then
			echo "1"
		else
			echo "0"
		fi
	fi
}

last_devinfo_clear()
{
	LAST_ptype=""
	LAST_logsn65=""
	LAST_CUR_OS=""
	LAST_oscore=""
	LAST_cpustat_cpurate=""
	LAST_bootime=""
	LAST_cpustat_physmemM=""
	LAST_keyinfo_start=0
	LAST_keyinfo_end=0
	LAST_palog_nodecount=0
	LAST_keyinfo_freestart=0
	LAST_palog_cpucnt=0
	LAST_keyinfo_freeend=0
	LAST_palog_hardrate=0
	LAST_palog_hardsize=0
	LAST_palog_memrate=0
	LAST_softinfo_buildtime=""
	LAST_softinfo_ver==""
	LAST_softinfo_ostype=""
	LAST_clicfg_sshrc=0
	LAST_saasagent_ver=""

}

last_devinfo_save()
{
	echo -n "" > ${SAAS_DEVINFO_FILE}.all
	echo "1_0=${LAST_saasagent_ver}" >> ${SAAS_DEVINFO_FILE}.all
	echo "2_0=${LAST_ptype}" >> ${SAAS_DEVINFO_FILE}.all
	echo "3_0=${LAST_logsn65}" >> ${SAAS_DEVINFO_FILE}.all
	echo "4_0=${LAST_CUR_OS}" >> ${SAAS_DEVINFO_FILE}.all
	echo "5_0=${LAST_oscore}" >> ${SAAS_DEVINFO_FILE}.all
	echo "6_1=${LAST_cpustat_cpurate}" >> ${SAAS_DEVINFO_FILE}.all
	echo "7_1=${LAST_palog_cpucnt}" >> ${SAAS_DEVINFO_FILE}.all
	echo "8_1=${LAST_cpustat_physmemM}" >> ${SAAS_DEVINFO_FILE}.all
	echo "9_1=${LAST_palog_memrate}" >> ${SAAS_DEVINFO_FILE}.all
	echo "10_1=${LAST_palog_hardsize}" >> ${SAAS_DEVINFO_FILE}.all
	echo "11_1=${LAST_palog_hardrate}" >> ${SAAS_DEVINFO_FILE}.all
	echo "12_1=${LAST_keyinfo_start}" >> ${SAAS_DEVINFO_FILE}.all
	echo "13_1=${LAST_keyinfo_end}" >> ${SAAS_DEVINFO_FILE}.all
	echo "14_1=${LAST_keyinfo_freestart}" >> ${SAAS_DEVINFO_FILE}.all
	echo "15_1=${LAST_keyinfo_freeend}" >> ${SAAS_DEVINFO_FILE}.all
	echo "16_1=${LAST_palog_nodecount}" >> ${SAAS_DEVINFO_FILE}.all
	echo "17_0=${LAST_softinfo_buildtime}" >> ${SAAS_DEVINFO_FILE}.all
	echo "18_0=${LAST_softinfo_ver}" >> ${SAAS_DEVINFO_FILE}.all
	echo "19_0=${LAST_softinfo_ostype}" >> ${SAAS_DEVINFO_FILE}.all
	echo "20_0=${LAST_clicfg_sshrc}" >> ${SAAS_DEVINFO_FILE}.all
	echo "21_1=${LAST_bootime}" >> ${SAAS_DEVINFO_FILE}.all
}

devinfo_get()
{
	[ "$1" = "getall" ] && last_devinfo_clear

	local logcpustat=`${PALOGEYE} cpu stat`
	# mylog "devinfo>cpustat ${logcpustat}"
	if echo "${logcpustat}" | grep -q "invalid alen or shmid\|can't connect to"; then
		mylog "devinfo>cpustat cmd failed."
		logcpustat=""
	elif ! echo "${logcpustat}" | grep -q "="; then
		mylog "devinfo>cpustat cmd return failed"
		logcpustat=""
	else
		for kv in ${logcpustat}; do
			k=`echo ${kv} | cut -d'=' -f1`
			if [ "${k}" = "pahwkeyno" -o "${k}" = "cpurate" -o \
				"${k}" = "temperature" -o "${k}" = "freememM" -o \
				"${k}" = "physmemM" -o "${k}" = "hwproduct" -o \
				"${k}" = "cpulist" ]; then
				eval "cpustat_${kv}"
			fi
		done
	fi
	
	# "$cpu_count $cpuper $memsize $memper $hardsize $hardper"
	baseinfo_row=`get_basic_info`
	# "$systype $keyid $start $end $upgrade_start $upgrade_end $nodecount"
	sysinfo_row=`get_sys_info`
	# "$BUILDTIME $SOFTVER $OSTYPE $CORETYPE"
	softinfo_row=`get_soft_info`

	#1_0, 1表示TLV的Tag类型，0表示这个Tag类型的值是字符串还是整形，0表示字符串. 1表示整型
	echo -n "" > ${SAAS_DEVINFO_FILE}

	#1.
	if [ "${LAST_saasagent_ver}" = "" ]; then
		LAST_saasagent_ver=`cat ${LOGROOT}/app/${APP_NAME}/app.inf | grep '^app_version=' | cut -d'=' -f2`
		if [ "${LAST_saasagent_ver}" != "" ]; then
			echo "1_0=${LAST_saasagent_ver}" >> ${SAAS_DEVINFO_FILE}
		fi
	fi

	#2. Get product type
	ptype=`echo ${sysinfo_row}|awk '{print $1}'`
	if [ "${ptype}" != "" ]; then 
		echo "2_0=${ptype}" >> ${SAAS_DEVINFO_FILE}
		LAST_ptype=${ptype}
	fi
	
	#3. Get machine code(83 bytes)
	local logsn65=`${PALOGEYE} key get`
	if ! expr "${logsn65}" : "^[a-fA-F0-9]\{65\}$" > /dev/null; then
		mylog "devinfo>keyget Unexpected! last=${LAST_logsn65}"
		logsn65=""
	fi
	if [ "${logsn65}" != "" -a "${LAST_logsn65}" != "${logsn65}" ]; then 
		echo "3_0=${logsn65}" >> ${SAAS_DEVINFO_FILE}
		LAST_logsn65=${logsn65}
	fi

	#4. Get Operation system platform
	if [ "${LAST_CUR_OS}" != "${CUR_OS}" ]; then 
		echo "4_0=${CUR_OS}" >> ${SAAS_DEVINFO_FILE}
		LAST_CUR_OS=${CUR_OS}
	fi

	#5. Get Operation system core version
	oscore=`uname -r`
	if [ "${LAST_oscore}" != "${oscore}" ]; then
		echo "5_0=${oscore}" >> ${SAAS_DEVINFO_FILE}
		LAST_oscore=${oscore}
	fi

	#6. Get cpu rate
	if [ "${cpustat_cpurate}" != "" -a "${LAST_cpustat_cpurate}" != "${cpustat_cpurate}" ]; then 
		echo "6_1=${cpustat_cpurate}" >> ${SAAS_DEVINFO_FILE}
		LAST_cpustat_cpurate=${cpustat_cpurate}
	fi
	# row 0
	palog_cpucnt=`echo ${baseinfo_row}|awk '{print $1}'`
	if [ "${palog_cpucnt}" != "" -a "${LAST_palog_cpucnt}" != "${palog_cpucnt}" ]; then
		echo "7_1=${palog_cpucnt}" >> ${SAAS_DEVINFO_FILE}
		LAST_palog_cpucnt=${palog_cpucnt}
	fi

	#8. Get physical memory size
	if [ "${cpustat_physmemM}" != "" -a "${LAST_cpustat_physmemM}" != "${cpustat_physmemM}" ]; then 
		echo "8_1=${cpustat_physmemM}" >> ${SAAS_DEVINFO_FILE}
		LAST_cpustat_physmemM=${cpustat_physmemM}
	fi
	# row 3
	palog_memrate=`echo ${baseinfo_row}|awk '{print $4}'`
	if [ "${palog_memrate}" != "" -a "${LAST_palog_memrate}" != "${palog_memrate}" ]; then
		echo "9_1=${palog_memrate}" >> ${SAAS_DEVINFO_FILE}
		LAST_palog_memrate=${palog_memrate}
	fi 

	# row 4
	palog_hardsize=`echo ${baseinfo_row}|awk '{print $5}'`
	if [ "${palog_hardsize}" != "" -a "${LAST_palog_hardsize}" != "${palog_hardsize}" ]; then
		echo "10_1=${palog_hardsize}" >> ${SAAS_DEVINFO_FILE}
		LAST_palog_hardsize=${palog_hardsize}
	fi
	# row 5
	palog_hardrate=`echo ${baseinfo_row}|awk '{print $6}'`
	if [ "${palog_hardrate}" != "" -a "${LAST_palog_hardrate}" != "${palog_hardrate}" ]; then
		echo "11_1=${palog_hardrate}" >> ${SAAS_DEVINFO_FILE}
		LAST_palog_hardrate=${palog_hardrate}
	fi

	#13. Get license start row 2
	keyinfo_start=`echo ${sysinfo_row}|awk '{print $3}'`
	if [ "${keyinfo_start}" != "" -a "${LAST_keyinfo_start}" != "${keyinfo_start}" ]; then
		echo "12_1=${keyinfo_start}" >> ${SAAS_DEVINFO_FILE}
		LAST_keyinfo_start=${keyinfo_start}
	fi

	#14. Get license end row 3
	keyinfo_end=`echo ${sysinfo_row}|awk '{print $4}'`
	if [ "${keyinfo_end}" != "" -a "${LAST_keyinfo_end}" != "${keyinfo_end}" ]; then 
		echo "13_1=${keyinfo_end}" >> ${SAAS_DEVINFO_FILE}
		LAST_keyinfo_end=${keyinfo_end}
	fi

	# row 4
	keyinfo_freestart=`echo ${sysinfo_row}|awk '{print $5}'`
	if [ "${keyinfo_freestart}" != "" -a "${LAST_keyinfo_freestart}" != "${keyinfo_freestart}" ]; then
		echo "14_1=${keyinfo_freestart}" >> ${SAAS_DEVINFO_FILE}
		LAST_keyinfo_freestart=${keyinfo_freestart}
	fi

	#18. Get license upgrade end row 5
	keyinfo_freeend=`echo ${sysinfo_row}|awk '{print $6}'`
	if [ "${keyinfo_freeend}" != "" -a "${LAST_keyinfo_freeend}" != "${keyinfo_freeend}" ]; then
		echo "15_1=${keyinfo_freeend}" >> ${SAAS_DEVINFO_FILE}
		LAST_keyinfo_freeend=${keyinfo_freeend}
	fi

	# panalog node count row 6
	palog_nodecount=`echo ${sysinfo_row}|awk '{print $7}'`
	if [ "${palog_nodecount}" != "" -a "${LAST_palog_nodecount}" != "${palog_nodecount}" ]; then
		echo "16_1=${palog_nodecount}" >> ${SAAS_DEVINFO_FILE}
		LAST_palog_nodecount=${palog_nodecount}
	fi

	# row 0
	softinfo_buildtime=`echo ${softinfo_row}|awk '{print $1}'`
	if [ "${softinfo_buildtime}" != "" -a "${LAST_softinfo_buildtime}" != "${softinfo_buildtime}" ]; then
		echo "17_0=${softinfo_buildtime}" >> ${SAAS_DEVINFO_FILE}
		LAST_softinfo_buildtime=${softinfo_buildtime}
	fi
	# row 1
	softinfo_ver=`echo ${softinfo_row}|awk '{print $2}'`
	if [ "${softinfo_ver}" != "" -a "${LAST_softinfo_ver}" != "${softinfo_ver}" ]; then
		echo "18_0=${softinfo_ver}" >> ${SAAS_DEVINFO_FILE}
		LAST_softinfo_ver=${softinfo_ver}
	fi
	# 19. Linux3.x row 2 & row 3
	ostype0=`echo ${softinfo_row}|awk '{print $3}'`
	ostype1=`echo ${softinfo_row}|awk '{print $4}'`
	softinfo_ostype=${ostype0}${ostype1}
	if [ "${softinfo_ostype}" != "" -a "${LAST_softinfo_ostype}" != "${softinfo_ostype}" ]; then
		echo "19_0=${softinfo_ostype}" >> ${SAAS_DEVINFO_FILE}
		LAST_softinfo_ostype=${softinfo_ostype}
	fi

	# Is sshd service Enable
	clicfg_sshrc=`sshd_status`
	if [ "${clicfg_sshrc}" != "" -a "${LAST_clicfg_sshrc}" != "${clicfg_sshrc}" ]; then
		echo "20_0=${clicfg_sshrc}" >> ${SAAS_DEVINFO_FILE}
		LAST_clicfg_sshrc=${clicfg_sshrc}
	fi

	#21. Get OS boot timestamp
	if [ "${CUR_OS}" = "FreeBSD" ]; then
		bootime=`sysctl kern.boottime | awk -F 'sec = ' '{print $2}' | cut -d',' -f1`
	else
		bootdate=`uptime -s`
		bootime=`date -d"${bootdate}" +%s`
	fi
	if [ "${bootime}" != "" -a "${LAST_bootime}" != "${bootime}" ]; then 
		echo "21_1=${bootime}" >> ${SAAS_DEVINFO_FILE}
		LAST_bootime=${bootime}
	fi

	last_devinfo_save
}

keyno_get()
{
	local info
	local hwkeyno
	local sfkeyno

	sfkeyno=""
	#use software keyno first
	local version=`${PALOGEYE} receiver stat | grep version | awk -F "=" '{print $2}'`
    if [ ${version} -eq 1 ]; then
        info=$(${PALOGEYE} key stat | grep 'keyno=')
		if [ "${info}" != "" ]; then
			sfkeyno=`echo ${info} | cut -d'=' -f2`
		fi
	else
		sfkeyno=`${PALOGEYE} key get ver=free|awk '{print $2}'`
    fi

	if [ "${sfkeyno}" != "" ]; then
		KEYNO="${sfkeyno}"
		return 0
	fi

	if [ "${LAST_KEYNO}" != "" ]; then
		KEYNO="${LAST_KEYNO}"
		return 0
	fi

	[ ${DEBUG} -eq 1 ] && mylog "Keyno not found"
	return 1
}

task_get()
{
	local domain_ip=""
	local res=""

	rs_str=`echo ${saas_taskget_url} | cut -d'/' -f3`
	[ "`echo ${rs_str} | grep ':'`" = "" ] && rs_str="${rs_str}:443"
	domain=`echo ${rs_str} | cut -d':' -f1`
	if [ "${domain}" != "${SAAS_DOMAIN}" ]; then
		domain_ip=`/usr/logd/www/app/saas_logclient/bin/nslookup_domain ${domain} ${SAAS_DNSSERVER}`
	else
		domain_ip=${SAAS_IP}
	fi

        res=$(${CURL} --resolve ${rs_str}:${domain_ip} --connect-timeout 5 -s -k -m 30 -X GET "${saas_taskget_url}?vkey=${saas_vkey}&lic=${KEYNO}&taskid=${saas_taskid}" | sed 's/\\//g' | tr -d '{}[]"')
	if [ "`echo ${res} | grep 'code:0'`" = "" ]; then
		echo "TASK_GET_RESP_ERROR:${res}"
		return 1
	fi
	datastr=$(echo ${res} | awk -F 'data:' '{print $2}')	
	TASK_TYPE=$(echo ${datastr} | awk -F 'tasktype:' '{print $2}' | cut -d ',' -f1)
	TASK_ARGSTR=$(echo ${datastr} | awk -F 'cmdstr:' '{print $2}')

	if [ "${TASK_TYPE}" = "" -o "${TASK_ARGSTR}" = "" ]; then
		echo "TASK_TYPE or TASK_ARGSTR not found"
		return 1
	fi

	if [ "`echo ${TASK_TYPE} | grep '^[a-zA-Z0-9]\+$'`" = "" ]; then
		echo "INVALID TASK_TYPE:${TASK_TYPE}"
		return 1
	fi

	return 0
}

# 返连的代码，还不知道怎么实现，先保留
webport_get()
{
	local res=""
	local httpd=""
	local webport=""
	local conf=""
	local line=""

	httpd=`ps -axw | grep 'nginx ' | grep -v grep | grep -v cluster| grep -v cloud|grep master`
	if [ "${httpd}" = "" ]; then
		mylog "Get webport nginx not running"
		return 1
	fi
	mylog "Get webport ${httpd}"

	webport=`echo ${httpd} | grep '\-p ' | awk -F '-p ' '{print $2}' | cut -d' ' -f1`
	if [ "${webport}" != "" ]; then
		if echo "${webport}" | grep -q "^[0-9]*$"; then
			echo "${webport}"
			return 0
		fi
		mylog "Get webport from cmd failed"
		return 1
	fi

	conf=`echo ${httpd} | awk -F '-c ' '{print $2}'|awk '{print $1}'`
	if [ "${conf}" = "" ] || [ ! -e "${conf}" ]; then
		mylog "Get webport from conf not exists"
		return 1
	fi
	mylog "Get webport ${conf}"

	while read line
	do
		line=$(echo "$line" | sed 's/^[ \t]*//')
		line=$(echo ${line}|grep -v "^#")
		if [ -z "$line" ]; then
			continue
		fi

		res=`echo $line |grep "listen"|grep "ssl"|grep -v grep`
		if [ "${res}" != "" ]; then
			webport=$(echo "$line" |awk '{print $2}')
		fi
	done < ${conf}

	echo "${webport}"
	return 0
}

sshdport_get()
{
	if [ "${CUR_OS}" = "FreeBSD" ]; then
		sshdport=`cat /etc/ssh/sshd_config | grep '^Port ' | awk -F 'Port ' '{print $2}' | cut -d' ' -f1`
		if [ "${sshdport}" = "" ]; then 
			echo "22"
			return 0
		fi
	elif [ "${CUR_OS}" = "Anolis" -o "${CUR_OS}" = "Euler" -o "${CUR_OS}" = "CentOS" ]; then
		# sshdport=`cat /etc/ssh/sshd_config | grep '^Port ' | awk -F 'Port ' '{print $2}' | cut -d' ' -f1`
		# if [ "${sshdport}" = "" ]; then 
		# 	echo "22"
		# 	return 0
		# fi
		sshdport=`netstat -tlnp | grep sshd | head -n 1 | awk '{print $4}' | cut -d':' -f2`
		if [ "${sshdport}" = "" ]; then
			echo "22"
			return 0
		fi
	elif [ "${CUR_OS}" = "Arm" ]; then
		dropbear=`ps -axw | grep 'dropbear ' | grep -v grep | head -n 1`
		if [ "${dropbear}" = "" ]; then 
			echo "22"
			return 0
		fi
		sshdport=`echo ${dropbear} | grep '\-p ' | awk -F '-p ' '{print $2}' | cut -d' ' -f1`
		if [ "${sshdport}" = "" ]; then 
			echo "22"
			return 0
		fi
	else
		#Linux
		sshdport=`netstat -tlnp | grep dropbear | head -n 1 | awk '{print $4}' | cut -d':' -f2`
		if [ "${sshdport}" = "" ]; then
			echo "22"
			return 0
		fi
	fi

	if echo "${sshdport}" | grep -q "^[0-9]*$"; then
		echo "${sshdport}"
		return 0
	fi
	return 1
}

sshrc_isenable()
{
	if [ -e ${CONF_FILE} ]; then
		echo "`cat ${CONF_FILE} | grep 'SSH_ENABLE=' | cut -d'=' -f2`"
	else
		echo "0"
	fi
}

sshrc_passfree()
{
	if [ -e ${CONF_FILE} ]; then
		echo "`cat ${CONF_FILE} | grep 'SSHRC_PASSWDFREE=' | cut -d'=' -f2`"
	else
		echo "0"
	fi
}

reverse_conn()
{
	# 返连的代码，还不知道怎么实现，先保留
	local req=$1
	local sshpass="${LOGROOT}/app/${APP_NAME}/bin/sshpass"
	local autossh="${LOGROOT}/app/${APP_NAME}/bin/autossh"
	local web_req=0

	for line in ${req};
	do
		line=`echo ${line} | sed 's/,/ /g'`
		for kv in ${line};
		do
			eval ${kv}
		done

		#do ssh reverse connection
		if [ "${tgtport}" = "web" ]; then
			localport=`webport_get`
			[ $? -eq 1 ] && continue
			web_req=1

		elif [ "${tgtport}" = "ssh" ]; then
			enable=`sshrc_isenable`
			[ "${enable}" = "" -o "${enable}" = "0" ] && continue
			#start sshd
			sshd_enable 'on'

			localport=`sshdport_get`
			[ $? -eq 1 ] && continue
		else
			localport=${tgtport}
		fi

		# 30分钟后隧道断开
		export AUTOSSH_MAXLIFETIME=1800
		#export AUTOSSH_MAXSTART=5
		export AUTOSSH_LOGLEVEL=0
		sshstr=" -p \"panabit\" -s ${autossh} -M 0 -o \"StrictHostKeyChecking no\" -y -N -R ${gwport}:localhost:${localport} ${saas_rc_account}@${SAAS_IP} -p ${sshd_port}"

		# killall autossh
		mylog "autossh ${AUTOSSH_MAXLIFETIME}"
		mylog "${sshstr}"
		eval "${sshpass} ${sshstr} &"
	
	done

	local passwdfree=`sshrc_passfree`
	if [ ${web_req} -eq 1 -a "${passwdfree}" == "1" ]; then
		palogkey=`${PALOGEYE} key get tag=session|awk '{print $1}'`
		[ "${palogkey}" != "" ] && keylen=`echo ${palogkey}|awk '{print length($0)}'` || keylen=0
		if [ ${keylen} -eq 32 ]; then
			SAAS_BODY="str=22=${palogkey}" #loginkey's tlv type is 22
			SAAS_MSGTYPE=7	#reverse connection response
			curtm=`date +%s`
			resp=`${SAAS_PING} ${SAAS_IP} ${SAAS_PORT} "SLOG" ${SAAS_MSGTYPE} ${SSPING_VER} ${curtm} ${PANAOS_ACTIVE} ${KEYNO} ${SAAS_BODY}`
			SAAS_BODY="null"
			SAAS_MSGTYPE=1	#ping 
		fi
	fi
}

for pid in `ps -axw | grep ${TASK_EXEC_SCRIPT} | grep -v grep | awk '{print $1}'`
do
	kill -9 ${pid} >/dev/null 2>&1
done
#Start task exec script in daemon
${TASK_EXEC_SCRIPT} ${TASK_DIR} ${TASK_SCRIPT_PATH} &

PKTCNT=0
SSPING_VER=4
SAAS_MSGTYPE=1	#ping request
SLEEP_INTERVAL=5
SAAS_BODY="null"
while true;
do
	[ ${DEBUG} -eq 1 ] && echo "ready sleep 5"
	sleep ${SLEEP_INTERVAL}

	#每隔3小时请求一次新的saasip
	if [ `expr ${PKTCNT} % $(( 10800 / ${SLEEP_INTERVAL}))` -eq 0 ]; then 
		SAAS_IP=`/usr/logd/www/app/saas_logclient/bin/nslookup_domain "${SAAS_DOMAIN}" ${SAAS_DNSSERVER}`
		mylog "Get saas ip0 ${SAAS_DOMAIN} -> ${SAAS_IP}"
		[ $? -eq 1 ] && continue
	fi

	#每隔10分钟取一次keyno
	if [ `expr ${PKTCNT} % $((600 / ${SLEEP_INTERVAL}))` -eq 0 -o "${KEYNO}" == "" ]; then 
		keyno_get
		mylog "Get keyno ${KEYNO}"
		[ $? -eq 1 ] && continue
		if [ "${KEYNO}" != "" -a "${LAST_KEYNO}" != "${KEYNO}" ]; then
			#key changed, we should clear cached information.
			LAST_KEYNO=${KEYNO}
			last_devinfo_clear
		fi
	fi

	if [ ${DEBUG} -eq 1 ]; then 
		mylog "keyno=${KEYNO}"
		mylog "saas_ip=${SAAS_IP}"
	fi

	if [  "${SAAS_IP}" == "" ]; then 
		SAAS_IP=`/usr/logd/www/app/saas_logclient/bin/nslookup_domain "${SAAS_DOMAIN}" ${SAAS_DNSSERVER}`
		mylog "Get saas ip1 ${SAAS_DOMAIN} -> ${SAAS_IP}"
	fi

	PKTCNT=$((${PKTCNT} + 1))

	# #检查PANAOS是否正常运行中
	PANAOS_ACTIVE=1
	# [ "`${PALOGEYE} key info | grep 't connect to'`" != "" ] && PANAOS_ACTIVE=0

	#发送消息并接收SaaS服务器返回的请求.
	curtm=`date +%s`
	mylog "${SAAS_PING} ${SAAS_IP} ${SAAS_PORT} ${KEYNO} ${SAAS_BODY}"
	resp=`${SAAS_PING} ${SAAS_IP} ${SAAS_PORT} "SLOG" ${SAAS_MSGTYPE} ${SSPING_VER} ${curtm} ${PANAOS_ACTIVE} ${KEYNO} ${SAAS_BODY}`
	[ "${resp}" = "" ] && continue
	resp_msgtype=${resp%%:*}
	resp_msgbody=`echo ${resp} | cut -d':' -f2`
	mylog "mstype ${resp_msgtype} ${resp_msgbody}"
	if [ "${resp_msgtype}" = "ERROR" ]; then
		mylog "###saasping response error:${resp}"
		last_devinfo_clear
		continue
	fi
	#创建一个文件以标记已成功与saas server连接
	echo "${curtm}" > ${SAAS_SERVER_ONLINETAG_FILE}

	SAAS_BODY="null"
	SAAS_MSGTYPE=1	#ping 
	case ${resp_msgtype} in
	"3") 	
		#DEVINFO_REQUEST
		devinfo_get ${resp_msgbody}
		if [ -e ${SAAS_DEVINFO_FILE} ]; then
			SAAS_BODY="file=${SAAS_DEVINFO_FILE}"
			SAAS_MSGTYPE=5	#devinfo response
		fi
		continue
		;;

	"4")
		#TASK_REQUEST
		;;

	"6")
		#REVERSE CONNECTION REQUEST
		reverse_conn ${resp#*6:}
		continue
		;;

	"10")
		#set sleep interval
		[ "`echo "${resp#*10:}" | grep '^[0-9]\+$'`" != "" ] && SLEEP_INTERVAL=${resp#*10:}
		continue
		;;

	*)
		continue
		;;
	esac

	#检查任务参数是否合规
	for tag in ${resp#*4:}
	do
		tag="saas_${tag}"
		eval ${tag}
	done

	if [ "`echo ${saas_taskget_url} | grep -E "^https://${WHITE_DOMAIN1}[:\/]|^https://${WHITE_DOMAIN2}[:\/]|^https://${WHITE_DOMAIN3}[:\/]|^https://${WHITE_DOMAIN4}[:\/]"`" = "" ]; then 
		mylog "INVALID_TASKGET_URL: ${saas_taskget_url}"
		continue
	fi

	if [ "`echo ${saas_taskreport_url} | grep -E "^https://${WHITE_DOMAIN1}[:\/]|^https://${WHITE_DOMAIN2}[:\/]|^https://${WHITE_DOMAIN3}[:\/]|^https://${WHITE_DOMAIN4}[:\/]"`" = "" ]; then 
		mylog "INVALID_TASKREPORT_URL: ${saas_taskreport_url}"
		continue
	fi

	if [ "`echo "${saas_taskid}" | grep '^[0-9]\+$'`" = "" ]; then
		mylog "INVALID_TASKID: ${saas_taskid}"
		continue
	fi

	if [ "`echo "${saas_vkey}" | grep '^[a-zA-Z0-9]\+$'`" = "" ]; then
		mylog "INVALID_VKEY: ${saas_vkey}"
		continue
	fi

	if [ ${DEBUG} -eq 1 ]; then
		mylog "taskid:${saas_taskid}"
		mylog "vkey:${saas_vkey}"
		mylog "taskget_url:${saas_taskget_url}"
		mylog "taskreport_url:${saas_taskreport_url}"
	fi

	mylog "pull task..."
	#拉取任务
	task_get
	[ $? -eq 1 ] && continue
	if [ ${DEBUG} -eq 1 ]; then
		mylog "TASK_TYPE:${TASK_TYPE}"
		mylog "TASK_ARGSTR:${TASK_ARGSTR}"
	fi

	#检查任务脚本是否存在
	scriptname=${TASK_SCRIPT_PATH}${TASK_TYPE}
	if [ ! -e ${scriptname} ]; then
		task_report 1 "Task script not found"
		mylog "Task script not found"
		continue
	fi
	[ ! -x ${scriptname} ] && chmod +x ${scriptname}

	#将任务写成文件，taskexec.sh脚本会对其进行处理
	TASK_ARGSTR=$(echo ${TASK_ARGSTR} | sed 's/ /\\n/g')
	task_name="${TASK_DIR}${TASK_TYPE}_${curtm}_${saas_vkey}.saastask"
	echo -e "DEBUG=${DEBUG}\ncommon_file=${COMMON_FILE}\nsaas_serverip=${SAAS_IP}\nsaas_domain=${SAAS_DOMAIN}\nsaas_taskreport_url=${saas_taskreport_url}\nsaas_keyno=${KEYNO}\nsaas_vkey=${saas_vkey}\nsaas_taskid=${saas_taskid}\n${TASK_ARGSTR}" > ${task_name}
	chmod 777 ${task_name}
done
